using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace dreamex
{
    class NameTable
    {
        string _pakName;
        List<FileEntry> entries;

        public string pakName { get { return _pakName; } }
        public List<FileEntry> entryBase { get { return entries; } }

        public NameTable(string filename)
        {
            entries = new List<FileEntry>();
            _pakName = filename;

            using (BinaryReader br = new BinaryReader(File.Open(filename, FileMode.Open)))
            {
                // read magic
                byte[] buffer = new byte[12];            
                br.Read(buffer,0,12);

                int fileCount = br.ReadInt32();
                int numCount = br.ReadInt32();
                int byteCount = br.ReadInt32();
                
                char[] nameBlock = new char[byteCount];
                int[] lenBlock = new int[numCount];

                // read contents
                for (int i=0;i<fileCount;i++)
                    entries.Add(new FileEntry(br));
                for (int i=0;i<byteCount;i++)
                    nameBlock[i] = Tools.hex2char(br.ReadByte());
                for (int i=0;i<numCount;i++)
                    lenBlock[i] = br.ReadInt32();

                // assoziate entries
                foreach (FileEntry e in entries)
                    e.fillIn(nameBlock);
            }
        }

        public FileEntry findFile(string path)
        {
            return findFile(path.ToLower(), "", 0);
        }

        private FileEntry findFile(string pathLeft, string pathSofar, int offset)
        {
            int num = Tools.char2hex(pathLeft[0]);
            if (num < 0)
                return null;
            num += offset;
            if (num >= entries.Count)
                return null;

            string partial = pathLeft[0] + entries[num].partial;            
            if (!pathLeft.StartsWith(partial))
                return null;

            pathSofar += partial;
            pathLeft = pathLeft.Substring(partial.Length);
            if (pathSofar.Length != entries[num].nameLen+1)
                return null;

            if (entries[num].isRealFile())
            {
                if (pathLeft.Length != 0)
                    return null;
                return entries[num];
            }
            if (pathLeft.Length < 1)
                return null;
            return findFile(pathLeft, pathSofar, entries[num].nameOffset);
        }

        public void nameGuess()
        {
            nameGuess("", 0);
        }

        private void nameGuess(string nameSoFar, int offset)
        {
            for (int ch = 0; ch < 44; ch++)
            {
                if (Tools.hex2char(ch) == '?')
                    continue;

                string cs = "";
                cs += Tools.hex2char(ch);
                int num = ch + offset;
                
                if (num < entries.Count)
                {
                    string spec = nameSoFar + cs + entries[num].partial;
                    if (spec.Length == entries[num].nameLen + 1)
                    {
                        if (!entries[num].isRealFile())
                            nameGuess(spec, entries[num].nameOffset);
                        else if (!entries[num].used && Tools.isPath(spec.Replace('\\', '/')))
                            entries[num].addPath(spec);

                        //if (entries[num].isRealFile() && !Tools.isPath(spec.Replace('\\', '/')))
                            //entries[num].allPathz.Add(spec);
                    }
                }
            }
        }

    }

    class FileEntry
    {
        int offset, size;
        int hOffset, hLen, hRef;
        string partialName="";
        bool alreadyUsed=false;
        bool multiPath=false;
        string guessedPath="";
        public List<string> allPathz = new List<string>();

        public FileEntry(BinaryReader file)
        {
            offset = file.ReadInt32();
            size = file.ReadInt32();
            hOffset = file.ReadInt32();
            hLen = file.ReadInt32();
            hRef = file.ReadInt32();
            if (isRealFile())
                hLen--;
        }
        public string partial { get { return partialName; } }
        public int nameLen { get { return hLen; } }
        public string guess { get { if (multiPath) return ""; else return guessedPath; } }
        public int fileOffset { get { return offset; } }
        public int fileLen { get { return size; } }
        public bool used { get { return alreadyUsed; } }
        public int nameOffset { get { return hOffset; } }

        public void fillIn(char[] table)
        {
            for (int i = hRef; i < table.Length; i++)
            {
                if (table[i] == 0)
                    break;
                partialName += table[i];
            }            
        }
        public void addPath(string path)
        {
            if (guessedPath == "")
                guessedPath = path;
            else
                multiPath = true;
        }

        public void markUsed()
        {
            alreadyUsed = true;
        }
        public bool isRealFile()
        {
            return size > 0;
        }                        
    }
}
