using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace dreamex
{
    class MEntry{
        public string name;
        public uint len;
        public MEntry(uint l, string n) { name = n; len = l; }
    }

    class PakXtract
    {

        const int blocksize = 1024*1024;
        byte[] buffer;

        string path;
        List<NameTable> nameTables = null;
        int doubleUsed = 0, foundNum = 0, guessedNum = 0, notFoundNum = 0;
        public StreamWriter log;

        public PakXtract(string path)
        {
            this.path = path;
            buffer = new byte[blocksize];
            nameTables = new List<NameTable>();
            Console.WriteLine("rehashing filenames...");
            foreach (string file in Directory.GetFiles(path))
                if (file.EndsWith(".pak"))
                    nameTables.Add(new NameTable(file));
            log = new StreamWriter("log.txt");
        }

        public void extractFiles(string indexFile)
        {            
            Console.WriteLine("locating indexed filenames\n");
            
            using(StreamReader sr = new StreamReader(indexFile))
            {
                FileEntry entry;
                string pak;
                int count = 0;
                while (!sr.EndOfStream)
                {
                    string line = sr.ReadLine();
                    if (line.Length > 0)
                        if (locate(line, out entry, out pak))
                        {
                            foundNum++;            
                            extract(entry, pak, line);
                        }
                    
                    count++; if ((count % 1000) == 0) Console.WriteLine("...@file {0}...",count);
                }
            }
            Console.WriteLine("\nguessing remaining filenames using hash-search\n");

            StreamWriter sw = new StreamWriter("log2.txt");
            foreach (NameTable nt in nameTables)
            {
                SortedDictionary<uint, MEntry> dict = new SortedDictionary<uint, MEntry>();            
                Console.WriteLine("processing {0}...",nt.pakName);
                nt.nameGuess();
                sw.WriteLine("\n{0}\n",Path.GetFileName(nt.pakName));
                foreach (FileEntry entry in nt.entryBase)
                {
                    if (!entry.used && entry.isRealFile() && entry.guess != "")
                    {
                        guessedNum++;
                        extract(entry, nt.pakName, entry.guess);
                    }
                    else if (entry.isRealFile() && !entry.used)
                    {
                        //Console.WriteLine("unresolved entry");
                        //foreach (string f in entry.allPathz)
                        //    Console.WriteLine(f);
                        //Console.ReadKey();
                    }
                    if (entry.isRealFile() && entry.guess != null && entry.guess != "")
                    {
                        dict.Add((uint)entry.fileOffset, new MEntry((uint)entry.fileLen, entry.guess));
                    }
                }
                
                foreach (uint off in dict.Keys)
                {
                    sw.WriteLine("{0:X8} - {1:X8}({2}) : {3}", off, dict[off].len + off, dict[off].len,dict[off].name);
                }
            }
            sw.Close();
        }

        public bool locate(string file, out FileEntry entry, out string pak)
        {
            pak = "";
            entry = null;
            foreach (NameTable table in nameTables)
            {
                entry = table.findFile(file);
                if (entry != null && !entry.used)
                {
                    pak = table.pakName;
                    entry.addPath (file);
                    entry.markUsed();
                    return true;
                }
                else if (entry != null && entry.used)
                {
                    Console.WriteLine("double ref : " + file);
                    doubleUsed++;
                    return false;
                }
            }
            Console.WriteLine("not found : " + file);
            notFoundNum++;
            return false;
        }

        private void extract(FileEntry entry, string pak, string file)
        {
            if (file.EndsWith(".png") || file.EndsWith(".wav")) return; 
            // normalize & create path
            string filename = path + "\\" + file.Replace('/', '\\');
            string dir = filename.Substring(0, filename.LastIndexOf('\\'));
            if (!Directory.Exists(dir))
                Directory.CreateDirectory(dir);

            log.WriteLine("{1}:{0}, {2:X8}-{3:X8}", file, Path.GetFileNameWithoutExtension(pak), entry.fileOffset, entry.fileOffset + entry.fileLen);
                
            if (!File.Exists(filename))
            {
                // extract
                using (FileStream br = new FileStream(pak, FileMode.Open))
                using (FileStream bw = new FileStream(filename, FileMode.Create))
                {
                    br.Seek(entry.fileOffset, SeekOrigin.Begin);
                    int len2write = entry.fileLen;
                    int reqLen = 0, bytesRead = 0;
                    while (len2write > 0)
                    {
                        reqLen = Math.Min(blocksize, len2write);
                        bytesRead = br.Read(buffer, 0, reqLen);
                        len2write -= bytesRead;
                        bw.Write(buffer, 0, bytesRead);
                    }
                }
            }
        }

        public int countDoubleUsed { get { return doubleUsed; } }
        public int countFound { get { return foundNum; } }
        public int countGuessed { get { return guessedNum; } }
        public int countNotFound { get { return notFoundNum; } }

        public int countTotal
        {
            get {
                int count = 0;
                foreach (NameTable table in nameTables)
                    foreach (FileEntry entry in table.entryBase)
                        if (entry.isRealFile())
                            count++;
                return count; }
        }
    }
}
