using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace dreamex
{
    class Indexer
    {
        Dictionary<string, int> paths = null;
        const int blocksize = 256*1024;
        const int minLen = 9;

        public int numFiles { get { return paths.Count; } }

        public void run(string dir)
        {
            paths = new Dictionary<string, int>();
            foreach(string file in Directory.GetFiles(dir))
            {
                if (file.EndsWith(".pak"))
                {
                    Console.WriteLine("cdr indexing {0}...", file);
                    locateCdr(file);
                }
            }
            Console.WriteLine("indexing bundles...");
            locateBun(dir);
        }

        public void writeIndex(string file)
        {
            using (StreamWriter sw = new StreamWriter(file))
            {
                foreach (string entry in paths.Keys)
                    sw.WriteLine(entry);
            }
        }

        private void locateCdr(string file)
        {
            const string magic = "shark3d_snake_binary";
            byte[] fileId = new byte[magic.Length];
            NameTable nt = new NameTable(file);

            using (FileStream br = new FileStream(file, FileMode.Open))
            foreach (FileEntry entry in nt.entryBase)
            {
                if (entry.isRealFile() && entry.fileLen > magic.Length)
                {
                    br.Seek(entry.fileOffset, SeekOrigin.Begin);
                    br.Read(fileId, 0, magic.Length);
                    if (Tools.compare(magic, fileId))
                    {
                        byte[] block = new byte[entry.fileLen];
                        br.Seek(entry.fileOffset, SeekOrigin.Begin);
                        br.Read(block, 0, entry.fileLen);
                        processCdr(block);
                    }
                }
            }
        }

        private void locateBun(string dir)
        {
            PakXtract locator = new PakXtract(dir);
            
            // copy .bun file list
            List<string> bundles = new List<string>();
            foreach (string file in paths.Keys)
                if (file.EndsWith(".bun"))
                    bundles.Add(file);

            foreach (string file in bundles)            
            {
                string pak;
                FileEntry entry;
                if (locator.locate(file, out entry, out pak))
                    processBun(pak, entry);
            }
        }

        private void processCdr(byte[] block)
        {
            string curPath = "";
    
            for (int i = 0; i < block.Length; i++)
            {
                char c = (char)block[i];
                if (c == '\0' || c == '#')
                {
                    if (curPath.Length > minLen && Tools.isPath(curPath))
                        normalizeAdd(curPath);
                }
                if (Tools.isAllowed(c))
                    curPath += c;
                else
                    curPath = "";
            }
        }

        private void processBun(string pak, FileEntry entry)
        {
            using (BinaryReader br = new BinaryReader(File.Open(pak,FileMode.Open)))
            {
                br.BaseStream.Seek(entry.fileOffset, SeekOrigin.Begin);
                br.ReadBytes(3);
                int count = Tools.switchEndian(br.ReadInt16());
                br.ReadBytes(2);
                for (int i = 0; i < count; i++)
                {
                    int len = Tools.switchEndian(br.ReadInt16());
                    normalizeAdd(new String(br.ReadChars(len)));
                }
            }            
        }

        private void normalizeAdd(string file)
        {
            file = file.Replace('/', '\\');
            if (file[0] == '\\') file = file.Substring(1);

            if (!paths.ContainsKey(file))
                paths.Add(file, 0);
            //Console.WriteLine(curPath);
        }
    }
}
