using System;
using System.IO;
using System.Collections.Generic;

namespace dreamex
{
    public static class Tools
    {
        private static List<string> allowedExt;
        private const string allowedChars = "-/._abcdefghijklmnopqrstuvwxyz1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ'";
        private const string charTable = "\0abcdefghijklmnopqrstuvwxyz\\??-_'.0123456789";            

        static Tools()
        {
            allowedExt = new List<string>();
            using (StreamReader sr = new StreamReader("ext.txt"))
            {
                string line;
                while (!sr.EndOfStream && (line = sr.ReadLine()) != "")
                    allowedExt.Add(line);
            }
        }

        public static bool testFileExt(string path)
        {
            string[] parts = path.Split('.');
            string ext = parts[parts.Length-1];
            return allowedExt.Contains(ext);
        }

        public static char hex2char(int a)
        {
            if (a > charTable.Length)
                return '?';
            return charTable[a];
        }

        public static int char2hex(char c)
        {
            return charTable.IndexOf(c);
        }
        
        public static bool isAllowed(char c)
        {
            return allowedChars.Contains(Char.ToLower(c).ToString());
        }

        public static bool isPath(string path)
        {
            int idxPoint = path.IndexOf('.');
            if (idxPoint < 0 || idxPoint != path.LastIndexOf('.'))
                return false;
            if (path.IndexOf('/') < 0)
                return false;
            if (path.Contains("//"))
                return false;
            if (!Tools.testFileExt(path))
                return false;

            return true;
        }
        
        public static bool compare(string s, byte[] a)
        {
            if (s.Length != a.Length)
                return false;
            for (int i = 0; i < s.Length; i++)
                if ((byte)s[i] != a[i])
                    return false;
            return true;
        }

        public static int switchEndian(int num)
        {
            return (num >> 8) + ((num & 0xff) << 8);
        }
    }
}
